@@ -1,4 +1,4 @@
-/* $OpenBSD: crl.c,v 1.11 2018/02/07 05:47:55 jsing Exp $ */
+/* $OpenBSD: crl.c,v 1.15 2021/10/31 16:47:27 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -88,7 +88,7 @@ static struct {
 	int verify;
 } crl_config;
 
-static struct option crl_options[] = {
+static const struct option crl_options[] = {
 	{
 		.name = "CAfile",
 		.argname = "file",
@@ -223,9 +223,9 @@ crl_main(int argc, char **argv)
 	int ret = 1, i;
 	BIO *out = NULL;
 	X509_STORE *store = NULL;
-	X509_STORE_CTX ctx;
+	X509_STORE_CTX *ctx = NULL;
 	X509_LOOKUP *lookup = NULL;
-	X509_OBJECT xobj;
+	X509_OBJECT *xobj = NULL;
 	EVP_PKEY *pkey;
 	const EVP_MD *digest;
 	char *digest_name = NULL;
@@ -243,7 +243,7 @@ crl_main(int argc, char **argv)
 		}
 	}
 
-	digest = EVP_sha1();
+	digest = EVP_sha256();
 
 	memset(&crl_config, 0, sizeof(crl_config));
 	crl_config.informat = FORMAT_PEM;
@@ -281,6 +281,8 @@ crl_main(int argc, char **argv)
 
 	if (crl_config.verify) {
 		store = X509_STORE_new();
+		if (store == NULL)
+			goto end;
 		lookup = X509_STORE_add_lookup(store, X509_LOOKUP_file());
 		if (lookup == NULL)
 			goto end;
@@ -298,20 +300,26 @@ crl_main(int argc, char **argv)
 			    X509_FILETYPE_DEFAULT);
 		ERR_clear_error();
 
-		if (!X509_STORE_CTX_init(&ctx, store, NULL, NULL)) {
+		if ((ctx = X509_STORE_CTX_new()) == NULL)
+			goto end;
+		if ((xobj = X509_OBJECT_new()) == NULL)
+			goto end;
+
+		if (!X509_STORE_CTX_init(ctx, store, NULL, NULL)) {
 			BIO_printf(bio_err,
 			    "Error initialising X509 store\n");
 			goto end;
 		}
-		i = X509_STORE_get_by_subject(&ctx, X509_LU_X509,
-		    X509_CRL_get_issuer(x), &xobj);
+		i = X509_STORE_get_by_subject(ctx, X509_LU_X509,
+		    X509_CRL_get_issuer(x), xobj);
 		if (i <= 0) {
 			BIO_printf(bio_err,
 			    "Error getting CRL issuer certificate\n");
 			goto end;
 		}
-		pkey = X509_get_pubkey(xobj.data.x509);
-		X509_OBJECT_free_contents(&xobj);
+		pkey = X509_get_pubkey(X509_OBJECT_get0_X509(xobj));
+		X509_OBJECT_free(xobj);
+		xobj = NULL;
 		if (!pkey) {
 			BIO_printf(bio_err,
 			    "Error getting CRL issuer public key\n");
@@ -429,10 +437,9 @@ crl_main(int argc, char **argv)
 	BIO_free_all(bio_out);
 	bio_out = NULL;
 	X509_CRL_free(x);
-	if (store) {
-		X509_STORE_CTX_cleanup(&ctx);
-		X509_STORE_free(store);
-	}
+	X509_STORE_CTX_free(ctx);
+	X509_STORE_free(store);
+	X509_OBJECT_free(xobj);
 
 	return (ret);
 }
